@@ -11,7 +11,6 @@ High-level interface for translating `metrics.yaml` into other formats.
 from pathlib import Path
 import os
 import shutil
-import sys
 import tempfile
 from typing import Any, Callable, Dict, Iterable, List
 
@@ -29,8 +28,10 @@ class Outputter:
     Class to define an output format.
 
     Each outputter in the table has the following member values:
+
     - output_func: the main function of the outputter, the one which
       does the actual translation.
+
     - clear_patterns: A list of glob patterns to clear in the directory before
       writing new results to it.
     """
@@ -67,10 +68,12 @@ def translate_metrics(
     :param output_dir: the path to the output directory
     :param translation_func: the function that actually performs the translation.
         It is passed the following arguments:
+
             - metrics_objects: The tree of metrics as pings as returned by
               `parser.parse_objects`.
             - output_dir: The path to the output directory.
             - options: A dictionary of output format-specific options.
+
         Examples of translation functions are in `kotlin.py` and `swift.py`.
     :param clear_patterns: a list of glob patterns of files to clear before
         generating the output files. By default, no files will be cleared (i.e.
@@ -80,17 +83,16 @@ def translate_metrics(
     :param parser_config: A dictionary of options that change parsing behavior.
         See `parser.parse_metrics` for more info.
     """
+    input_filepaths = util.ensure_list(input_filepaths)
+
+    if lint.glinter(input_filepaths, parser_config):
+        return 1
+
     all_objects = parser.parse_objects(input_filepaths, parser_config)
 
     if util.report_validation_errors(all_objects):
         return 1
 
-    if lint.lint_metrics(all_objects.value, parser_config):
-        print(
-            "NOTE: These warnings will become errors in a future release of Glean.",
-            file=sys.stderr,
-        )
-
     # allow_reserved is also relevant to the translators, so copy it there
     if parser_config.get("allow_reserved"):
         options["allow_reserved"] = True
