@@ -10,6 +10,7 @@ import shutil
 
 from glean_parser import parser
 from glean_parser import translate
+from glean_parser.util import load_yaml_or_json
 
 import util
 
@@ -25,7 +26,7 @@ def test_translate_unknown_format():
 
 
 def test_translate_missing_directory(tmpdir):
-    output = Path(tmpdir) / "foo"
+    output = Path(str(tmpdir)) / "foo"
 
     translate.translate(
         ROOT / "data" / "core.yaml",
@@ -38,7 +39,7 @@ def test_translate_missing_directory(tmpdir):
 
 
 def test_translate_remove_obsolete_kotlin_files(tmpdir):
-    output = Path(tmpdir) / "foo"
+    output = Path(str(tmpdir)) / "foo"
 
     translate.translate(
         ROOT / "data" / "core.yaml",
@@ -55,7 +56,7 @@ def test_translate_remove_obsolete_kotlin_files(tmpdir):
 
 
 def test_translate_retains_existing_markdown_files(tmpdir):
-    output = Path(tmpdir) / "foo"
+    output = Path(str(tmpdir)) / "foo"
 
     translate.translate(
         ROOT / "data" / "core.yaml",
@@ -66,7 +67,7 @@ def test_translate_retains_existing_markdown_files(tmpdir):
 
     # Move the file to a different location, translate always writes to
     # metrics.md.
-    shutil.move(output / "metrics.md", output / "old.md")
+    shutil.move(str(output / "metrics.md"), str(output / "old.md"))
 
     assert len(list(output.iterdir())) == 1
 
@@ -117,3 +118,47 @@ def test_translate_send_in_pings(tmpdir):
     assert objs["baseline"]["counter"].send_in_pings == ["metrics"]
     assert objs["baseline"]["event"].send_in_pings == ["events"]
     assert objs["baseline"]["c"].send_in_pings == ["custom", "metrics"]
+
+
+def test_translate_dont_remove_extra_files(tmpdir):
+    output = Path(str(tmpdir)) / "foo"
+    output.mkdir()
+
+    with (output / "extra.txt").open("w") as fd:
+        fd.write("\n")
+
+    translate.translate(
+        ROOT / "data" / "core.yaml",
+        "kotlin",
+        output,
+        parser_config={"allow_reserved": True},
+    )
+
+    assert len(list(output.iterdir())) == 6
+    assert "extra.txt" in [str(x.name) for x in output.iterdir()]
+
+
+def test_external_translator(tmpdir):
+    tmpdir = Path(str(tmpdir))
+
+    def external_translator(all_objects, output_dir, options):
+        assert {"foo": "bar", "allow_reserved": True} == options
+
+        for category in all_objects:
+            with (tmpdir / category).open("w") as fd:
+                for metric in category:
+                    fd.write("{}\n".format(metric))
+
+    translate.translate_metrics(
+        ROOT / "data" / "core.yaml",
+        Path(str(tmpdir)),
+        external_translator,
+        options={"foo": "bar"},
+        parser_config={"allow_reserved": True},
+    )
+
+    content = load_yaml_or_json(ROOT / "data" / "core.yaml")
+
+    expected_keys = set(content.keys()) - set(["$schema"])
+
+    assert set(x.name for x in tmpdir.iterdir()) == expected_keys
